#ifndef TRAINER__H
#define TRAINER__H

#include "ReadingMachine.hpp"
#include "ConfigDataset.hpp"
#include "SubConfig.hpp"
#include "TestNetwork.hpp"

class Trainer
{
  private :

  using Dataset = ConfigDataset;
  using DataLoader = std::unique_ptr<torch::data::StatelessDataLoader<torch::data::datasets::MapDataset<Dataset, torch::data::transforms::Stack<torch::data::Example<> > >, torch::data::samplers::RandomSampler>, std::default_delete<torch::data::StatelessDataLoader<torch::data::datasets::MapDataset<Dataset, torch::data::transforms::Stack<torch::data::Example<> > >, torch::data::samplers::RandomSampler> > >;

  private :

  ReadingMachine & machine;
  DataLoader dataLoader{nullptr};
  std::unique_ptr<torch::optim::Adam> denseOptimizer;
  std::unique_ptr<torch::optim::SparseAdam> sparseOptimizer;
  std::size_t epochNumber{0};
  int batchSize{100};
  int nbExamples{0};

  public :

  Trainer(ReadingMachine & machine);
  void createDataset(SubConfig & goldConfig);
  float epoch();

};

#endif
